@@ -3,8 +3,9 @@
 # modify it under the same terms as Perl itself.
 
 package Convert::ASN1;
-
-# $Id: ASN1.pm,v 1.27 2003/05/12 17:45:57 gbarr Exp $
+{
+  $Convert::ASN1::VERSION = '0.23';
+}
 
 use 5.004;
 use strict;
@@ -14,6 +15,8 @@ use Exporter;
 use constant CHECK_UTF8 => $] > 5.007;
 
 BEGIN {
+  local $SIG{__DIE__};
+  eval { require bytes and 'bytes'->import };
 
   if (CHECK_UTF8) {
     require Encode;
@@ -21,7 +24,6 @@ BEGIN {
   }
 
   @ISA = qw(Exporter);
-  $VERSION = '0.17';
 
   %EXPORT_TAGS = (
     io    => [qw(asn_recv asn_send asn_read asn_write asn_get asn_ready)],
@@ -42,12 +44,13 @@ BEGIN {
   $EXPORT_TAGS{all} = \@EXPORT_OK;
 
   @opParts = qw(
-    cTAG cTYPE cVAR cLOOP cOPT cCHILD cDEFINE
+    cTAG cTYPE cVAR cLOOP cOPT cEXT cCHILD cDEFINE
   );
 
   @opName = qw(
     opUNKNOWN opBOOLEAN opINTEGER opBITSTR opSTRING opNULL opOBJID opREAL
-    opSEQUENCE opSET opUTIME opGTIME opUTF8 opANY opCHOICE opROID
+    opSEQUENCE opSET opUTIME opGTIME opUTF8 opANY opCHOICE opROID opBCD
+    opEXTENSIONS
   );
 
   foreach my $l (\@opParts, \@opName) {
@@ -186,6 +189,15 @@ sub registeroid {
   $self->{oidtable}{$oid}=$handler;
 }
 
+sub registertype {
+   my $self = shift;
+   my $def = shift;
+   my $type = shift;
+   my $handler = shift;
+
+   $self->{options}{handlers}{$def}{$type}=$handler;
+}
+
 # In XS the will convert the tree between perl and C structs
 
 sub _pack_struct { $_[0] }
@@ -207,7 +219,7 @@ sub encode {
 
 
 # Encode tag value for encoding.
-# We assume that the tag has been correclty generated with asn_tag()
+# We assume that the tag has been correctly generated with asn_tag()
 
 sub asn_encode_tag {
   $_[0] >> 8
@@ -256,7 +268,7 @@ sub decode {
 	0,
 	length $_[0], 
 	undef,
-	[],
+	{},
 	$_[0]);
 
     $result;
