@@ -1,10 +1,11 @@
-# Copyright (c) 2000-2002 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 2000-2005 Graham Barr <gbarr@pobox.com>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
 package Convert::ASN1;
-
-# $Id: Debug.pm,v 1.7 2002/01/22 11:24:28 gbarr Exp $
+{
+  $Convert::ASN1::VERSION = '0.23';
+}
 
 ##
 ## just for debug :-)
@@ -126,7 +127,7 @@ sub asn_dump {
       };
 
       /^(?:(OBJECT ID)|(RELATIVE-OID))/ && do {
-	my @op; $op[opTYPE] = $1 ? opOBJID : opROID;
+	my @op; $op[cTYPE] = $1 ? opOBJID : opROID;
 	Convert::ASN1::_dec_object_id({},\@op,{},$tmp,$_[0],$pos,$len);
 	printf " = %s\n",$tmp;
         last;
