@@ -1,7 +1,11 @@
+# Copyright (c) 2000-2005 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# This program is free software; you can redistribute it and/or
+# modify it under the same terms as Perl itself.
 
 package Convert::ASN1;
-
-# $Id: //depot/asn/lib/Convert/ASN1/IO.pm#13 $
+{
+  $Convert::ASN1::VERSION = '0.23';
+}
 
 use strict;
 use Socket;
@@ -109,8 +113,6 @@ sub asn_read { # $fh, $buffer, $offset
   # we have enough to decode a tag and a length. We then know
   # how many more bytes to read
 
-  my $pos = 0;
-  my $need = 0;
   if ($_[2]) {
     if ($_[2] > length $_[1]) {
       require Carp;
@@ -122,26 +124,31 @@ sub asn_read { # $fh, $buffer, $offset
   else {
     $_[1] = '';
   }
+
+  my $pos = 0;
+  my $need = 0;
   my $depth = 0;
   my $ch;
   my $n;
-
-  while(($n = $need - length $_[1]) > 0) {
-    sysread($_[0],$_[1],$n,length $_[1]) or
-	goto READ_ERR;
-  }
+  my $e;
+  
 
   while(1) {
-    $need = $pos + 2;
-    my $tch = ord(substr($_[1],$pos++,1));
+    $need = ($pos + ($depth * 2)) || 2;
 
+    while(($n = $need - length $_[1]) > 0) {
+      $e = sysread($_[0],$_[1],$n,length $_[1]) or
+	goto READ_ERR;
+    }
+
+    my $tch = ord(substr($_[1],$pos++,1));
     # Tag may be multi-byte
     if(($tch & 0x1f) == 0x1f) {
       my $ch;
       do {
         $need++;
 	while(($n = $need - length $_[1]) > 0) {
-	  sysread($_[0],$_[1],$n,length $_[1]) or
+	  $e = sysread($_[0],$_[1],$n,length $_[1]) or
 	      goto READ_ERR;
 	}
 	$ch = ord(substr($_[1],$pos++,1));
@@ -151,7 +158,7 @@ sub asn_read { # $fh, $buffer, $offset
     $need = $pos + 1;
 
     while(($n = $need - length $_[1]) > 0) {
-      sysread($_[0],$_[1],$n,length $_[1]) or
+      $e = sysread($_[0],$_[1],$n,length $_[1]) or
 	  goto READ_ERR;
     }
 
@@ -165,7 +172,7 @@ sub asn_read { # $fh, $buffer, $offset
       $need = $pos + $len;
 
       while(($n = $need - length $_[1]) > 0) {
-	sysread($_[0],$_[1],$n,length $_[1]) or
+	$e = sysread($_[0],$_[1],$n,length $_[1]) or
 	    goto READ_ERR;
       }
 
@@ -181,19 +188,18 @@ sub asn_read { # $fh, $buffer, $offset
       $pos += $len;
     }
 
-    $need = $pos + 2*$depth;
-
-    while(($n = $need - length $_[1]) > 0) {
-      sysread($_[0],$_[1],$n,length $_[1]) or
-	  goto READ_ERR;
-    }
     last unless $depth;
   }
 
+  while(($n = $pos - length $_[1]) > 0) {
+    $e = sysread($_[0],$_[1],$n,length $_[1]) or
+      goto READ_ERR;
+  }
+
   return length $_[1];
 
 READ_ERR:
-    $@ = "I/O Error $! " . CORE::unpack("H*",$_[0]);
+    $@ = defined($e) ? "Unexpected EOF" : "I/O Error $!"; # . CORE::unpack("H*",$_[1]);
     return undef;
 }
 
