@@ -1,10 +1,11 @@
-# Copyright (c) 2000-2002 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 2000-2005 Graham Barr <gbarr@pobox.com>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
 package Convert::ASN1;
-
-# $Id: _encode.pm,v 1.18 2003/05/06 21:29:07 gbarr Exp $
+{
+  $Convert::ASN1::VERSION = '0.23';
+}
 
 BEGIN {
   unless (CHECK_UTF8) {
@@ -35,6 +36,7 @@ my @encode = (
   \&_enc_any,
   \&_enc_choice,
   \&_enc_object_id,
+  \&_enc_bcd,
 );
 
 
@@ -43,6 +45,7 @@ sub _encode {
   my $var;
 
   foreach my $op (@{$ops}) {
+    next if $op->[cTYPE] == opEXTENSIONS;
     if (defined(my $opt = $op->[cOPT])) {
       next unless defined $stash->{$opt};
     }
@@ -106,21 +109,27 @@ sub _enc_integer {
 sub _enc_bitstring {
 # 0      1    2       3     4     5      6
 # $optn, $op, $stash, $var, $buf, $loop, $path
+  my $vref = ref($_[3]) ? \($_[3]->[0]) : \$_[3];
+
+  if (CHECK_UTF8 and Encode::is_utf8($$vref)) {
+    utf8::encode(my $tmp = $$vref);
+    $vref = \$tmp;
+  }
 
   if (ref($_[3])) {
     my $less = (8 - ($_[3]->[1] & 7)) & 7;
-    my $len = ($_[3]->[1] + 7)/8;
+    my $len = ($_[3]->[1] + 7) >> 3;
     $_[4] .= asn_encode_length(1+$len);
     $_[4] .= chr($less);
-    $_[4] .= substr($_[3]->[0], 0, $len);
+    $_[4] .= substr($$vref, 0, $len);
     if ($less && $len) {
-      substr($_[4],-1) &= chr(0xff << $less);
+      substr($_[4],-1) &= chr((0xff << $less) & 0xff);
     }
   }
   else {
-    $_[4] .= asn_encode_length(1+length $_[3]);
+    $_[4] .= asn_encode_length(1+length $$vref);
     $_[4] .= chr(0);
-    $_[4] .= $_[3];
+    $_[4] .= $$vref;
   }
 }
 
@@ -359,6 +368,7 @@ sub _enc_any {
   my $handler;
   if ($_[1]->[cDEFINE] && $_[2]->{$_[1]->[cDEFINE]}) {
     $handler=$_[0]->{oidtable}{$_[2]->{$_[1]->[cDEFINE]}};
+    $handler=$_[0]->{handlers}{$_[1]->[cVAR]}{$_[2]->{$_[1]->[cDEFINE]}} unless $handler;
   }
   if ($handler) {
     $_[4] .= $handler->encode($_[3]);
@@ -388,5 +398,13 @@ sub _enc_choice {
 }
 
 
+sub _enc_bcd {
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
+  my $str = ("$_[3]" =~ /^(\d+)/) ? $1 : "";
+  $str .= "F" if length($str) & 1;
+  $_[4] .= asn_encode_length(length($str) / 2);
+  $_[4] .= pack("H*", $str);
+}
 1;
 
