@@ -34,5 +34,23 @@ sub btest ($$) {
   $_[1]
 }
 
+
+sub rtest ($$$) {
+  unless (eval { require Data::Dumper } ) {
+    print "ok $_[0] # skip need Data::Dumper\n";
+    return;
+  }
+
+  local $Data::Dumper::Sortkeys = 1;
+  my $ok = Data::Dumper::Dumper($_[1]) eq Data::Dumper::Dumper($_[2]);
+
+  unless ($ok) {
+    printf "#line %d %s\n",(caller)[2,1];
+    print "not ";
+  }
+  print "ok $_[0]\n";
+  $ok;
+}
+
 1;
 
